/*********************************************************************************
*
*               SHARP MZ-80B/80B2/2000/2200 Emulator for UNIX
*
*                        EmuZ-2000  Extend Board DLL
*
*                               << MZ-1R12 >>
*
*                          Main Module [MZ1R12.C]
*
*               Programmed & Porting by 2003 EmuZ-2000 (M.Y)
*		Bug fix and TF-Edition state save support by T.Fukui (2010)
*               28 Feb. 2013 New FileName support*
*		- When I/O base address is changed, the state load is ignored.
*		1th Feb. 2015 UNIX Version and add initialize 0xff values. by T.Fukui
*
*********************************************************************************/
//

#include "../config.h"

#include <stdio.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_SYS_STAT_H
#include <sys/stat.h>
#endif
#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif
#if defined(HAVE_STRING_H) || defined(WIN32) || defined(_WIN32)
#include <string.h>
#endif
#if defined(HAVE_STRINGS_H)
#include <strings.h>
#endif
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#include <limits.h>
#include <gtk/gtk.h>

#include "../interface/plugin_posix.h"
#include "../interface/emuz2000ioBoard.h"
#include "../interface/plugin_ex.h"

#define GLADEUIFILE	"mz1r12ui.xml"
#define UIGTK_IDCANCEL	-1
#define UIGTK_IDOK	0

////////////////////////////////////
// Prototype 
////////////////////////////////////
UCHAR
InportFunction(
	USHORT	port
	) ;

int
OutportFunction(
	USHORT	port,
	UCHAR	data
	) ;

////////////////////////////////////
// Global Variable 
////////////////////////////////////
EMUZ2000EBIH	mz1r12EBIH = {
	"31DD2499-F757-481b-9BCA-F0C60733368B",
	"MZ-1R12 by SHARP",
	"32KByte S-RAM Extend Board",
	"Copyright (C) 2003 EmuZ-2000, 2010-2013 S.Tsuneoka, 2010-2015 T.Fukui",
	"http://www5d.biglobe.ne.jp/~object",
	"emuz2000@excite.co.jp",
	0x00000000,//Reserved
	InportFunction,
	OutportFunction,
	0xF8,
	0x3,
	USE_INFODIALOG
} ;

USHORT	address = 0 ;
UCHAR	mz1r12Buf[32768] ;
DWORD	gId = 0 ;
DWORD	dwBase = 0 ;

////////////////////////////////////
// Extern
////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////
// Logic Start
//
///////////////////////////////////////////////////////
// Subroutines
static int fileutils_pathnormalize( char *destbuf, int destbufmax, const char *srcpath, const char *basepath )
{
	int len = 0, retval = 0;
	const char *cp = srcpath;
	char *cp2;

	if (*cp == '~') {
		char *pptr;
		if ((pptr = getenv("HOME")) == NULL)
			pptr = "";
		if (strlen(pptr) - 1 >= destbufmax) {
			*destbuf = '\0';
			return FALSE;
		}
		strcpy( destbuf, pptr );
		len = strlen( destbuf );
		if (destbuf[len - 1] != '\\' && destbuf[len - 1] != '/') {
			destbuf[len] = '/';
			destbuf[len + 1] = '\0';
			len++;
		}
		cp++;
		while (*cp == '/' || *cp == '\\')
			cp++;
	} else if (*cp != '\\' && *cp != '/' && basepath && strlen(basepath) > 0) {
		if (strlen(basepath) + 2 > destbufmax) {
			*destbuf = '\0';
			return FALSE;
		}
		strcpy( destbuf, basepath );
		len = strlen( destbuf );
		if (destbuf[len - 1] != '/' && destbuf[len - 1] != '\\') {
#ifndef WIN32
			destbuf[len]     = '/';
#else
			destbuf[len]     = '\\';
#endif
			destbuf[len + 1] = '\0';
			len++;
		}
	}
	while (*cp) {
		if (len >= destbufmax - 1) {
			destbuf[len] = '\0';
			retval = -1;
			break;
		}
		if (*cp >= ' ' && *cp != 0x7f) {
			destbuf[len] = *cp++;
			len++;
		}
	}
	destbuf[len] = '\0';
	for (cp2 = destbuf; *cp2; cp2++) {
#ifndef WIN32
		if (*cp2 == '\\')
			*cp2 = '/';
#else
		if (*cp2 == '/')
			*cp2 = '\\';
#endif
	}
	return retval;
}

///////////////////////////////////////////////////////
// window Proc.
static GtkRadioButton *radiobuttons[2];

int open_plugindialog( GtkWidget *uigtk_window )
{
	gint r;
	GtkBuilder *builder;
	GtkDialog *dialog;

	builder = gtk_builder_new();
	r = gtk_builder_add_from_file( builder, GLADEUIFILE, NULL );
	dialog = GTK_DIALOG( gtk_builder_get_object( builder, "mz1r12_dialog" ) );
#ifdef EMZ2000_PLGDIR
	if (/* r || */ !dialog) {
		r = gtk_builder_add_from_file( builder, EMZ2000_PLGDIR "/" GLADEUIFILE, NULL );
		dialog = GTK_DIALOG( gtk_builder_get_object( builder, "mz1r12_dialog" ) );
	}
#endif
#ifdef EMZ2000_LIBDIR
	if (/* r || */ !dialog) {
		r = gtk_builder_add_from_file( builder, EMZ2000_LIBDIR "/" GLADEUIFILE, NULL );
		dialog = GTK_DIALOG( gtk_builder_get_object( builder, "mz1r12_dialog" ) );
	}
#endif
	if (/* r || */ !dialog) {
		fprintf( stderr, GLADEUIFILE " and " EMZ2000_PLGDIR "/" GLADEUIFILE " and " EMZ2000_LIBDIR "/" GLADEUIFILE " not found or broken.\n" );
		g_object_unref( builder );
		return UIGTK_IDCANCEL;
	}
	gtk_builder_connect_signals( builder, NULL );
	//gtk_widget_set_parent_window( GTK_WIDGET(dialog), uigtk_window );

	radiobuttons[0] = GTK_RADIO_BUTTON( gtk_builder_get_object( builder, "radiobutton_adr1" ) );
	radiobuttons[1] = GTK_RADIO_BUTTON( gtk_builder_get_object( builder, "radiobutton_adr2" ) );
	switch (dwBase) {
	case 0x04 :
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(radiobuttons[0]), TRUE );
		break;
	default :
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(radiobuttons[1]), TRUE );
		break;
	}

	gtk_dialog_add_buttons( dialog,
		GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
		"DEFAULT", GTK_RESPONSE_NO, NULL );

retry:	r = gtk_dialog_run( GTK_DIALOG(dialog) );
	switch (r) {
	case GTK_RESPONSE_ACCEPT :
		if (gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(radiobuttons[0]) ))
			dwBase = 0x04;
		else
			dwBase = 0xf8;
		gtk_widget_destroy( GTK_WIDGET(dialog) );
		g_object_unref( builder );
		return UIGTK_IDOK;
	case GTK_RESPONSE_NO :
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(radiobuttons[1]), TRUE );
		goto retry;
	case GTK_RESPONSE_CANCEL :
	default : /* close button */
		break;
	}
	gtk_widget_destroy( GTK_WIDGET(dialog) );
	g_object_unref( builder );
	return UIGTK_IDCANCEL;
}

/////////////////////////////////////////////////////////////////////
// I/O Function Interface
/////////////////////////////////////////////////////////////////////
UCHAR
InportFunction(
	USHORT	port
	)
{
	UCHAR	data = 0 ;

	switch( port )
	{
	case	0x04:
	case	0xf8:
		address = 0 ;
		break ;
	case	0x05:
	case	0xf9:
		data = mz1r12Buf[address] ;
		address++ ;
		address &= 0x7fff ;
		break ;
	case	0x06:
	case	0xfa:
		break ;
	}
	return	data ;
}

int
OutportFunction(
	USHORT	port,
	UCHAR	data
	)
{
	switch( port )
	{
	case	0x04:
	case	0xf8:
		address &= 0xff ;
		address |= (USHORT)(data<<8) ;
		address &= 0x7fff ;
		break ;
	case	0x05:
	case	0xf9:
		address &= 0xff00 ;
		address |= (USHORT)data ;
		address &= 0x7fff ;
		break ;
	case	0x06:
	case	0xfa:
		mz1r12Buf[address] = data ;
		address++ ;
		address &= 0x7fff ;
		break ;
	}
	return	0 ;
}

/////////////////////////////////////////////////////////////////////
// EmuZ-2000 Interface
/////////////////////////////////////////////////////////////////////
BOOL
WINAPI
IntializeBoard( 
	VOID
	)
{
	int r;
	FILE *fp;
	DWORD dwReadSize ;
	char tfile[PATH_MAX];
	const char *fn = "~/.emz2000/mz_1r12.mem";

	memset( mz1r12Buf, 0xff, 32768 );
	r = fileutils_pathnormalize( tfile, PATH_MAX, fn, NULL );
	if (r < 0) {
		fprintf( stderr, "File read name initialize error (fn: %s)\n", fn );
		fp = NULL;
	} else
		fp = fopen( tfile, "r" );
	if (fp)	{
		dwReadSize = fread( mz1r12Buf, 1, 32768, fp );
		fclose( fp );
	} else
		fprintf( stderr, "File read open failed (fn: %s)\n", fn );

	dwBase = emuz2000_readProfileDword( gId, "BaseAddress" ) ;
	switch( dwBase )
	{
	case	0xf8:
	case	0x04:
		break ;
	default:
		dwBase = 0xf8 ;
		break ;
	}
	mz1r12EBIH.ioMapper.ucStartPort = (UCHAR)dwBase ;

	address = 0 ;
	return	TRUE ;
}

BOOL
WINAPI
ReIntializeBoard( 
	VOID
	)
{
	address = 0 ;
	return	TRUE ;
}


VOID
WINAPI
FinalizeBoard( 
	VOID 
	)
{
	int r;
	FILE *fp;
	DWORD dwWriteSize ;
	char tfile[PATH_MAX], tfile2[PATH_MAX];
	const char *fn = "~/.emz2000/mz_1r12.mem";
	const char *fn2 = "~/.emz2000";

	r = fileutils_pathnormalize( tfile, PATH_MAX, fn, NULL );
	if (r < 0) {
		fprintf( stderr, "File write name initialize error (fn: %s)\n", fn );
		fp = NULL;
	} else {
		fp = fopen( tfile, "w" );
		if (!fp) {
			r = fileutils_pathnormalize( tfile2, PATH_MAX, fn2, NULL );
			if (r >= 0)
				mkdir( tfile2, 0777U );
			fp = fopen( tfile, "w" );
		}
	}
	if (fp)	{
		dwWriteSize = fwrite( mz1r12Buf, 1, 32768, fp );
		fclose( fp );
	} else
		fprintf( stderr, "File write open failed (fn: %s)\n", fn );

	emuz2000_writeProfileDword( gId, "BaseAddress", dwBase ) ;
	return ;
}

VOID
WINAPI
ReportCpuClock( 
	int clk
	)
{
	return ;
}

VOID
WINAPI
InfomationDialog( 
	HWND	hWnd 
	)
{
	open_plugindialog( (GtkWidget *)hWnd );
	return ;
}

BOOL
WINAPI
BoardInfomationService( 
	PEMUZ2000EBIH *info,
	DWORD id
	)
{

	*info = &mz1r12EBIH ;
	gId = id ;
	return	TRUE ;
}

#ifdef EMUZ2000EX
BOOL WINAPI StateSaveOutputDataEx( int *ver, DWORD *size, PUCHAR *buf )
{
	static unsigned int data[2];

	data[0] = address;
	data[1] = dwBase;

	*size = sizeof(data);
	*buf = (void *)data;
	*ver = 1;				/* Data Format Ver.1 */
	return TRUE;
}

BOOL WINAPI StateSaveInputDataEx( int ver, DWORD size, PUCHAR buf )
{
	unsigned int *data = (void *)buf;
	unsigned int tdata[2] = { 0, 0 };	/* for automatic size calculation */

	if (ver > 1)				/* Newer Data Format Ver. */
		return FALSE;
	if (size != sizeof(tdata))
		return FALSE;
	if (dwBase != (DWORD)data[1])		/* I/O port changed */
		return FALSE;
	address    = (USHORT)data[0];
	return TRUE;
}
#else
BOOL
WINAPI
StateSaveOutputData( 
	DWORD *size,
	PUCHAR buf
	)
{
	*size = 0 ;
	buf = NULL ;
	return	TRUE ;
}

BOOL
WINAPI
StateSaveInputData( 
	DWORD size,
	PUCHAR buf
	)
{
	return	TRUE ;
}
#endif

/*
	Local Variables:
	mode:c++
	c-set-style:"k&r"
	c-basic-offset:4
	tab-width:4
	End:
*/
