/*********************************************************************************
*               SHARP MZ-80B/80B2/2000/2200 Emulator for UNIX
*                        EmuZ-2000  Extend Board DLL
*                               << PIO-3034 >>
*                          320KB Extension RAM Board
*                          Main Module [PIO-3034.C]
*
*                             Programmed by Suga
*        	      TF-Edition state save supported by T.Fukui
*                          UNIX Version by T.Fukui
*********************************************************************************/
//
// 【変更履歴】
// 2006年04月03日 Ver.0.1
//	作業者：Suga
//	内　容：mz_1r12 をベースに初版作成
//
// 2006年04月12日 Ver.1.0
//	作業者：Suga
//	内　容：ボード４枚までサポート
//　　　　　ＲＡＭデータ保存・復元のON/OFF、ベースアドレス、
//  　　　　ボード枚数設定ダイアログボックス追加
//
// 2010年04月20日 Ver.1.1
//	作業者：T.Fukui
//	内　容：TF-Editionステートセーブ対応、
//		bord_max -> BOARD_MAX
//
// 2013年02月28日 Ver.1.2
//	作業者：S.Tsuneoka
//	内　容：メモリファイル名変更
//		PIO-3034EMM*.MEM  pio_3034_emm*.mem
//
// 2015年01月30日 Ver.1.3
//	作業者：T.Fukui
//	内　容：メモリ初期化追加、ULONG -> UINTへ変更、UNIXへ移植
//		PIO-3034EMM*.MEM  pio_3034_emm*.mem
//

#include "config.h"

#include <stdio.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_SYS_STAT_H
#include <sys/stat.h>
#endif
#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif
#if defined(HAVE_STRING_H) || defined(WIN32) || defined(_WIN32)
#include <string.h>
#endif
#if defined(HAVE_STRINGS_H)
#include <strings.h>
#endif
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#include <limits.h>
#include <gtk/gtk.h>

#include "../interface/plugin_posix.h"
#include "../interface/emuz2000ioBoard.h"
#include "../interface/plugin_ex.h"
#include "uigtk.h"
#include "common.h"

#define GLADEUIFILE	"pio3034ui.xml"
GtkWidget *uigtk_window;

#define UINT	unsigned int

////////////////////////////////////
// Prototype 
////////////////////////////////////
UCHAR
InportFunction(
	USHORT	port
	) ;

int
OutportFunction(
	USHORT	port,
	UCHAR	data
	) ;

////////////////////////////////////
// Global Variable 
////////////////////////////////////
#define	RamFileDir	"~/.emz2000"
#define	RamFileNamefmt	RamFileDir "/pio_3034_emm%1.1d.mem"
#define	IniBaseAddress	"PIO-3034BaseAddress"
#define IniMaisuu		"PIO-3034Maisuu"
#define	IniRamSave		"PIO-3034RamSave"
#define	ram_size		0x50000
#define BOARD_MAX		4

EMUZ2000EBIH	pio3034EBIH = {
	"F69F0E1A-DB72-45e2-B698-77BEC81843E8",
	"PIO-3034 by I-O DATA",
	"320KB Extension RAM Board",
	"Copyright (C) 2006 Suga, 2010-2013 S.Tsuneoka, 2010-2015 T.Fukui",
	"http://www.ne.jp/asahi/suga/junkyard/",
	"suga@mar.email.ne.jp",
	0x00000000,	//Reserved
	InportFunction,
	OutportFunction,
	0,
	0,
	USE_INFODIALOG
} ;

UINT	address[BOARD_MAX];
UCHAR	ram_data[BOARD_MAX][ram_size];
DWORD	gId = 0;
UINT	IOBASE = 0;
UINT	MAISUU = 0;
UINT	RAM_SAVE = 0;

////////////////////////////////////
// Extern
////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////
// Logic Start
//
///////////////////////////////////////////////////////
// Subroutines
static int fileutils_pathnormalize( char *destbuf, int destbufmax, const char *srcpath, const char *basepath )
{
	int len = 0, retval = 0;
	const char *cp = srcpath;
	char *cp2;

	if (*cp == '~') {
		char *pptr;
		if ((pptr = getenv("HOME")) == NULL)
			pptr = "";
		if (strlen(pptr) - 1 >= destbufmax) {
			*destbuf = '\0';
			return FALSE;
		}
		strcpy( destbuf, pptr );
		len = strlen( destbuf );
		if (destbuf[len - 1] != '\\' && destbuf[len - 1] != '/') {
			destbuf[len] = '/';
			destbuf[len + 1] = '\0';
			len++;
		}
		cp++;
		while (*cp == '/' || *cp == '\\')
			cp++;
	} else if (*cp != '\\' && *cp != '/' && basepath && strlen(basepath) > 0) {
		if (strlen(basepath) + 2 > destbufmax) {
			*destbuf = '\0';
			return FALSE;
		}
		strcpy( destbuf, basepath );
		len = strlen( destbuf );
		if (destbuf[len - 1] != '/' && destbuf[len - 1] != '\\') {
#ifndef WIN32
			destbuf[len]     = '/';
#else
			destbuf[len]     = '\\';
#endif
			destbuf[len + 1] = '\0';
			len++;
		}
	}
	while (*cp) {
		if (len >= destbufmax - 1) {
			destbuf[len] = '\0';
			retval = -1;
			break;
		}
		if (*cp >= ' ' && *cp != 0x7f) {
			destbuf[len] = *cp++;
			len++;
		}
	}
	destbuf[len] = '\0';
	for (cp2 = destbuf; *cp2; cp2++) {
#ifndef WIN32
		if (*cp2 == '\\')
			*cp2 = '/';
#else
		if (*cp2 == '/')
			*cp2 = '\\';
#endif
	}
	return retval;
}

///////////////////////////////////////////////////////
// window Proc.
static GtkEntry *entry_adr;
static GtkRadioButton *radiobuttons_num[BOARD_MAX];
static GtkRadioButton *radiobuttons_save[2];

int open_plugindialog( GtkWidget *_uigtk_window )
{
	int i, t;
	char buf[LNMAX];
	gint r;
	const gchar *s;
	GtkBuilder *builder;
	GtkDialog *dialog;

	uigtk_window = _uigtk_window;
	builder = gtk_builder_new();
	r = gtk_builder_add_from_file( builder, GLADEUIFILE, NULL );
	dialog = GTK_DIALOG( gtk_builder_get_object( builder, "pio3034_dialog" ) );
#ifdef EMZ2000_PLGDIR
	if (/* r || */ !dialog) {
		r = gtk_builder_add_from_file( builder, EMZ2000_PLGDIR "/" GLADEUIFILE, NULL );
		dialog = GTK_DIALOG( gtk_builder_get_object( builder, "pio3034_dialog" ) );
	}
#endif
#ifdef EMZ2000_LIBDIR
	if (/* r || */ !dialog) {
		r = gtk_builder_add_from_file( builder, EMZ2000_LIBDIR "/" GLADEUIFILE, NULL );
		dialog = GTK_DIALOG( gtk_builder_get_object( builder, "pio3034_dialog" ) );
	}
#endif
	if (/* r || */ !dialog) {
		fprintf( stderr, GLADEUIFILE " and " EMZ2000_PLGDIR "/" GLADEUIFILE " and " EMZ2000_LIBDIR "/" GLADEUIFILE " not found or broken.\n" );
		g_object_unref( builder );
		return UIGTK_IDCANCEL;
	}
	gtk_builder_connect_signals( builder, NULL );
	//gtk_widget_set_parent_window( GTK_WIDGET(dialog), uigtk_window );

	entry_adr = GTK_ENTRY( gtk_builder_get_object( builder, "entry_adr" ) );
	snprintf( buf, LNMAX, "%02x", IOBASE );
	gtk_entry_set_text( entry_adr, buf );
	for (i = 0; i < BOARD_MAX; i++) {
		snprintf( buf, LNMAX, "radiobutton_num%d", i + 1 );
		radiobuttons_num[i] = GTK_RADIO_BUTTON( gtk_builder_get_object( builder, buf ) );
	}
	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(radiobuttons_num[MAISUU - 1]), TRUE );
	radiobuttons_save[0] = GTK_RADIO_BUTTON( gtk_builder_get_object( builder, "radiobutton_save1" ) );
	radiobuttons_save[1] = GTK_RADIO_BUTTON( gtk_builder_get_object( builder, "radiobutton_save2" ) );
	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(radiobuttons_save[RAM_SAVE - 1]), TRUE );

	gtk_dialog_add_buttons( dialog,
		GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
		"DEFAULT", GTK_RESPONSE_NO, NULL );

retry:	r = gtk_dialog_run( GTK_DIALOG(dialog) );
	switch (r) {
	case GTK_RESPONSE_ACCEPT :
		s = gtk_entry_get_text( entry_adr );
		if (sscanf( s, "%x", &t ) != 1 || t < 0 || t > 0xff) {
			uigtk_messagebox( "PIO-3034 I/O address require 2 characters hex number",
				"PIO-3034 Settings", UIGTK_BUTTON_OKONLY | UIGTK_ICON_ERROR );
			goto retry;
		}
		if (t % 4 != 0) {
			uigtk_messagebox( "PIO-3034 lower I/O address require only 0,4,8,c",
				"PIO-3034 Settings", UIGTK_BUTTON_OKONLY | UIGTK_ICON_ERROR );
			goto retry;
		}
		IOBASE = t;
		for (i = 0; i < BOARD_MAX; i++) {
			if (gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(radiobuttons_num[i]) )) {
				MAISUU = i + 1;
				break;
			}
		}
		if (i >= BOARD_MAX) {
			uigtk_messagebox( "PIO-3034 board numbers select error",
				"PIO-3034 Settings", UIGTK_BUTTON_OKONLY | UIGTK_ICON_ERROR );
			goto retry;
		}
	 	// 1 (もしくは 2 以外):行う 2:行わない
		if (gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(radiobuttons_save[1]) ))
			RAM_SAVE = 2;	// 行わない
		else
			RAM_SAVE = 1;	// 行う
		gtk_widget_destroy( GTK_WIDGET(dialog) );
		g_object_unref( builder );
		return UIGTK_IDOK;
	case GTK_RESPONSE_NO :
		snprintf( buf, LNMAX, "%02x", 0xa0U );
		gtk_entry_set_text( entry_adr, buf );
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(radiobuttons_num[4 - 1]), TRUE );
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(radiobuttons_save[1 - 1]), TRUE );
		goto retry;
	case GTK_RESPONSE_CANCEL :
	default : /* close button */
		break;
	}
	gtk_widget_destroy( GTK_WIDGET(dialog) );
	g_object_unref( builder );
	return UIGTK_IDCANCEL;
}

/////////////////////////////////////////////////////////////////////
// I/O Function Interface
/////////////////////////////////////////////////////////////////////
UCHAR
InportFunction(
	USHORT	port
	)
{
	UCHAR	data;
	UINT	ioport, bordno;

	bordno = (UINT)((port - IOBASE ) / 4);
	if(bordno >= MAISUU) return	0 ;
	ioport = (UINT)((port - IOBASE ) % 4);
	data = 0xff ;

	switch( ioport )
	{
	case	0x03:
		if (address[bordno] < ram_size)
			data = ram_data[bordno][ address[bordno] ] ;
		address[bordno]++ ;
		address[bordno] &= 0x7ffff ;
		break ;
	}
	return	data ;
}

int
OutportFunction(
	USHORT	port,
	UCHAR	data
	)
{
	UINT	ioport, bordno;

	bordno = (UINT)((port - IOBASE ) / 4);
	if(bordno >= MAISUU) return	0 ;
	ioport = (UINT)((port - IOBASE ) % 4);

	switch( ioport )
	{
	case	0x00:
		address[bordno] &= 0x7ff00 ;
		address[bordno] |= (UINT)data ;
		break ;
	case	0x01:
		address[bordno] &= 0x700ff ;
		address[bordno] |= (UINT)(data<<8) ;
		break ;
	case	0x02:
		address[bordno] &= 0x0ffff ;
		address[bordno] |= (UINT)(data<<16) ;
		address[bordno] &= 0x7ffff ;
		break ;
	case	0x03:
		if (address[bordno] < ram_size)
			ram_data[bordno][ address[bordno] ] = data;

		address[bordno]++ ;
		address[bordno] &= 0x7ffff ;
		break ;
	}
	return	0 ;
}

/////////////////////////////////////////////////////////////////////
// EmuZ-2000 Interface
/////////////////////////////////////////////////////////////////////
BOOL
WINAPI
IntializeBoard( 
	VOID
	)
{
	int r;
	FILE *fp;
	DWORD	dwReadSize;
	UINT	i;
	UINT	SKIP = FALSE;
	char	RamFileName[1024];
	char	tfile[PATH_MAX];

//	*--- ini RAMデータ保存・復元動作を読み出し
	RAM_SAVE = emuz2000_readProfileDword( gId, IniRamSave );
	switch( RAM_SAVE)
	{
	case	1:	// 行う
		break;
	case	2:	// 行わない
		break;
	default :	//	未設定 or 範囲外（初めて本DLLが使われたと判断する）
		RAM_SAVE = 1;
		IOBASE = 0xa0;
		MAISUU = 4;
		SKIP = TRUE;
	}

//	*--- ini からI/Oアドレス、使用ボード数を読み出し
	if(!SKIP)
	{
		IOBASE = emuz2000_readProfileDword( gId, IniBaseAddress ) & 0xfc;
		MAISUU = emuz2000_readProfileDword( gId, IniMaisuu );
		if (MAISUU < 1) MAISUU = 1;
		if (MAISUU > BOARD_MAX) MAISUU = BOARD_MAX;
	}

//	*--- EmuZ2000 本体へ使用I/O アドレス、ポート数を通知
	pio3034EBIH.ioMapper.ucStartPort = (UCHAR)IOBASE ;
	pio3034EBIH.ioMapper.nCount = (UCHAR)((MAISUU) * 4);

//	*--- ボードのアドレスカウンタ初期化
	for(i=0; i<BOARD_MAX; i++) address[i] = 0 ;

//	*--- ボードのメモリデータ初期化
	for (i = 0; i < BOARD_MAX; i++)
		memset( ram_data[i], 0xff, ram_size );

//	*--- ボードのセーブデータを復元
	for(i=0; i<MAISUU; i++)
	{
		sprintf(RamFileName, RamFileNamefmt, i);
		r = fileutils_pathnormalize( tfile, PATH_MAX, RamFileName, NULL );
		if (r < 0) {
			fprintf( stderr, "File read name initialize error (fn: %s)\n", RamFileName );
			fp = NULL;
		} else
			fp = fopen( tfile, "r" );
		if (fp)	{
			dwReadSize = fread( ram_data[i], 1, ram_size, fp );
			fclose( fp );
		} else
			fprintf( stderr, "File read open failed (fn: %s)\n", RamFileName );
	}

	return	TRUE;
}

BOOL
WINAPI
ReIntializeBoard( 
	VOID
	)
{
	UINT	i;

	for(i=0; i< BOARD_MAX; i++) address[i] = 0 ;
	return	TRUE ;
}


VOID
WINAPI
FinalizeBoard( 
	VOID 
	)
{
	int r;
	FILE *fp;
	DWORD dwWriteSize ;
	UINT	i;
	char	RamFileName[1024];
	char tfile[PATH_MAX], tfile2[PATH_MAX];

//	*--- ポードのデータをセーブ
	if(RAM_SAVE == 1)
	{
		for(i=0; i<MAISUU; i++) {
			sprintf(RamFileName, RamFileNamefmt, i);
			r = fileutils_pathnormalize( tfile, PATH_MAX, RamFileName, NULL );
			if (r < 0) {
				fprintf( stderr, "File write name initialize error (fn: %s)\n", RamFileName );
				fp = NULL;
			} else {
				fp = fopen( tfile, "w" );
				if (!fp) {
					r = fileutils_pathnormalize( tfile2, PATH_MAX, RamFileDir, NULL );
					if (r >= 0)
						mkdir( tfile2, 0777U );
					fp = fopen( tfile, "w" );
				}
			}	
			if (fp) {
				dwWriteSize = fwrite( ram_data[i], 1, ram_size, fp );
				fclose( fp );
			} else
				fprintf( stderr, "File write open failed (fn: %s)\n", RamFileDir );
		}
	}

//	*--- ini へボードのRAM保存・復元動作、I/Oアドレス、使用ボード数を保存
	emuz2000_writeProfileDword( gId, IniRamSave, RAM_SAVE );
	emuz2000_writeProfileDword( gId, IniBaseAddress, IOBASE ) ;
	emuz2000_writeProfileDword( gId, IniMaisuu, MAISUU ) ;

	return ;
}

VOID
WINAPI
ReportCpuClock( 
	int clk
	)
{
	return ;
}

VOID
WINAPI
InfomationDialog( 
	HWND	hWnd 
	)
{
	open_plugindialog( (GtkWidget *)hWnd );
	return ;
}

BOOL
WINAPI
BoardInfomationService( 
	PEMUZ2000EBIH *info,
	DWORD id
	)
{

	*info = &pio3034EBIH ;
	gId = id ;
	return	TRUE ;
}

#ifdef EMUZ2000EX
BOOL WINAPI StateSaveOutputDataEx( int *ver, DWORD *size, PUCHAR *buf )
{
	int i;
	static unsigned int data[BOARD_MAX];

	for (i = 0; i < BOARD_MAX; i++)
		data[i] = 0;
	*size = sizeof(data);
	*buf = (void *)data;
	*ver = 1;				/* Data Format Ver.1 */
	return TRUE;
}

BOOL WINAPI StateSaveInputDataEx( int ver, DWORD size, PUCHAR buf )
{
	int i;
	unsigned int *data = (void *)buf;
	unsigned int tdata[BOARD_MAX];		/* for automatic size calculation */

	for (i = 0; i < BOARD_MAX; i++)
		tdata[i] = 0;			/* for Compiler Warning */
	if (ver > 1)				/* Newer Data Format Ver. */
		return FALSE;
	if (size != sizeof(tdata))
		return FALSE;
	for (i = 0; i < BOARD_MAX; i++)
		address[i] = (UINT)data[i];
	return TRUE;
}
#else
BOOL
WINAPI
StateSaveOutputData( 
	DWORD *size,
	PUCHAR buf
	)
{
	*size = 0 ;
	buf = NULL ;
	return	TRUE ;
}

BOOL
WINAPI
StateSaveInputData( 
	DWORD size,
	PUCHAR buf
	)
{
	return	TRUE ;
}
#endif

/*
	Local Variables:
	mode:c++
	c-set-style:"k&r"
	c-basic-offset:4
	tab-width:4
	End:
*/
