/*********************************************************************************
*
*               SHARP MZ-80B/80B2/2000/2200 Emulator for UNIX
*
*                        EmuZ-2000  Extend Board DLL
*
*                               << MZ-1R13 >>
*
*                          Main Module [MZ1R13.C]
*
*               Programmed & Porting by 2003 EmuZ-2000 (M.Y)
*               Programmed & Porting by 2003-2006 Suga
*               TF-Edition state save support and modified by T.Fukui (2010-2016)
*               UNIX version modified by T.Fukui (2015-2016)
*
*********************************************************************************/
//
// 【変更履歴】
// 2003年02月16日 Ver.0.1
//	作業者：Suga
//	内　容：初版作成
//
// 2003年03月01日 Ver.0.2
//	作業者：Suga
//	内　容：漢字、辞書ファイルが存在しない場合、ROMイメージは全てFFで埋める。
//			漢字ファイル名にEmuZ-2500と同じ KANJI2.ROM も使えるように変更。
//			不要部分削除、コメント追加。
//
// 2003年03月05日 Ver.0.3
//	作業者：EmuZ-2000 (M.Y)
//	内　容：ビット上下入替処理変更 (if文の並びをforループに)
//
// 2003年03月06日 Ver.0.4
//	作業者：Suga
//	内　容：ビット上下間違い修正、不要コメント削除
//
// 2006年04月13日 Ver.0.41
//	作業者：Suga
//	内　容：URL,メールアドレス変更
//
// 2010年04月20日 Ver.0.5
//	作業者：T.Fukui
//	内　容：mz1R13Bankの初期化追加、TF-Edition ステートセーブサポート
//
// 2010年08月09日 Ver.0.51
//	作業者：T.Fukui
//	内　容：Ver0.5がVer.0.4ベースであったためVer.0.41ベースへ変更
//
// 2013年xx月xx日 Ver.0.6
//	作業者：S.Tsuneoka
//	内　容：参照ファイル名の変更
//
// 2015年01月29日 Ver.0.61
//	作業者：T.Fukui
//	内　容：UNIXへ移植とmz_1r13_kan25.romの削除
//
// 2016年02月26日 Ver.0.7
//	作業者：T.Fukui
//	内　容：ビット反転読み出しモードのバグ修正
//          (JET-2200Aの漢字ROMモードが正しく動作するようになる)
//
// 2016年04月13日 Ver.0.71
//	作業者：T.Fukui
//	内　容：構造体初期化データの修正等
//

#include "../config.h"

#include <stdio.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#if defined(HAVE_STRING_H) || defined(WIN32) || defined(_WIN32)
#include <string.h>
#endif
#if defined(HAVE_STRINGS_H)
#include <strings.h>
#endif
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#include <limits.h>

#include "../interface/plugin_posix.h"
#include "../interface/emuz2000ioBoard.h"
#include "../interface/plugin_ex.h"

#define ULONG	unsigned long

////////////////////////////////////
// Prototype 
////////////////////////////////////
UCHAR
InportFunction(
	USHORT	port
	) ;

int
OutportFunction(
	USHORT	port,
	UCHAR	data
	) ;

////////////////////////////////////
// Global Variable 
////////////////////////////////////
EMUZ2000EBIH	mz1R13EBIH = {
	"4F499AF5-B66D-44a6-9459-2C6F3EA87264",
	"MZ-1R13 by SHARP",
	"Kanji Rom Board",
	"Copyright (C) 2006 EmuZ-2000 & Suga, 2010-2013 S.Tsuneoka, 2010-2016 T.Fukui",
	"http://www.ne.jp/asahi/suga/junkyard/",
	//"http://www.cisnet.or.jp/home/suga/",
	"suga@mar.email.ne.jp",
	//"suga@po.cisnet.or.jp",
	0x00000000,//Reserved
	{ InportFunction,
	OutportFunction,
	0xb8,		// 開始I/Oアドレス
	0x4	},		// 使用アドレス数
	{ 0 /* USE_INFODIALOG */ }
} ;

USHORT	address = 0 ;			// ＲＯＭアドレス
UCHAR	mz1R13Kanji[131072] ;	// 漢字ＲＯＭ
UCHAR	mz1R13Dic[16384] ;		// 辞書ＲＯＭ
UCHAR	mz1R13Bank = 1;			// 辞書(0)・漢字(1)切替 (modified by T.Fukui)

DWORD	gId = 0 ;
DWORD	dwBase = 0 ;

////////////////////////////////////
// Extern
////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////
// Logic Start
//
/////////////////////////////////////////////////////////////////////
// I/O Function Interface
/////////////////////////////////////////////////////////////////////
UCHAR
InportFunction(
	USHORT	port
	)
{
	UCHAR	data = 0xff ;
	UCHAR	rom_data = 0xff ;
	ULONG	rom_address ;

	rom_address = (ULONG)address * 2 ;
	switch( port )
	{
	case	0xb8:
	case	0xba:
		if (mz1R13Bank == 1)	// フォント左側
		{
			rom_data = mz1R13Kanji[ rom_address ] ;
		} else					// 辞書ＲＯＭ１
		{
			rom_data = mz1R13Dic[ rom_address & 0x3fff ] ;
		}
		break ;
	case	0xb9:
	case	0xbb:
		if (mz1R13Bank == 1)	// フォント右側
		{
			rom_data = mz1R13Kanji[ rom_address + 1 ] ;
		} else					// 辞書ＲＯＭ２
		{
			rom_data = mz1R13Dic[ ( rom_address & 0x3fff ) + 1 ] ;
		}
		//
		// ＲＯＭアドレス下位８ビット側だけカウントアップ
		address = (USHORT)( ( address & 0xff00 ) + ( (address + 1) & 0x00ff) ) ;
		address %= 0xffff ;
		break ;
	}

	switch( port )
	{
	case	0xb8:	// 標準データ読み出し
	case	0xb9:	// ROM D0->D0,D1->D1,...D6->D6,D7->D7
		data = rom_data ;
		break ;
	case	0xba:	// ビット上下入替読み出し
	case	0xbb:	// ROM D0->D7,D1->D6,...D6->D1,D7->D0
		{			// if文からforループへ変更 EmuZ-2000 (M.Y) 2003年03月05日
			UCHAR	lop ;
			for( lop = 0, data = 0x00 ; lop < 8 ; lop++, rom_data <<= 1 ) {
				data >>= 1;						// 1bit消えるのを修正
				data |= ( rom_data & 0x80 );	// 上下間違い修正 2003年03月05日
			}
		}
		break ;
	}

	return	data ;
}

int
OutportFunction(
	USHORT	port,
	UCHAR	data
	)
{
	switch( port )
	{
	case	0xb8:	// ROMアドレス下位８ビット
		address &= 0xff00 ;
		address |= (USHORT)data ;
		address %= 0xffff ;
		break ;
	case	0xb9:	// ROMアドレス上位８ビット
		address &= 0xff ;
		address |= (USHORT)(data<<8) ;
		address %= 0xffff ;
		break ;
	case	0xba:	// 辞書(0)・漢字(1)ROM切替
		mz1R13Bank = (UCHAR)( data & 0x01 ) ;
		break;
	case	0xfa:
		break ;
	}
	return	0 ;
}

/////////////////////////////////////////////////////////////////////
// EmuZ-2000 Interface
/////////////////////////////////////////////////////////////////////
BOOL
WINAPI
IntializeBoard( 
	VOID
	)
{
	FILE *fp;
	DWORD dwReadSize ;

	//------*< 漢字ＲＯＭイメージファイル読み込み >*

	// ファイルが存在しない場合に備え全てFFで埋めとく 2003年03月01日
	memset(mz1R13Kanji, 0xff, 131072);

	// ファイル「1R13KAN.ROM」を読込用でオープン
#define FILENAME1 "mz_1r13_kan.rom"
	fp = fopen( FILENAME1, "r" );
#ifdef EMZ2000_PLGDIR
	if (!fp) {
		char tpath[PATH_MAX];
		strncpy( tpath, EMZ2000_PLGDIR "/", PATH_MAX );
		tpath[PATH_MAX - 1] = '\0';
		strncat( tpath, FILENAME1, PATH_MAX - strlen(tpath) - 1 );
		fp = fopen( tpath, "r" );
	}
#endif
#ifdef EMZ2000_LIBDIR
	if (!fp) {
		char tpath[PATH_MAX];
		strncpy( tpath, EMZ2000_LIBDIR "/", PATH_MAX );
		tpath[PATH_MAX - 1] = '\0';
		strncat( tpath, FILENAME1, PATH_MAX - strlen(tpath) - 1 );
		fp = fopen( tpath, "r" );
	}
#endif

#if 0
	if (!fp)	// オープン失敗
	{
			// ファイル「KANJI2.ROM」を読込用でオープン 2003年03月01日
			fp = fopen( "mz_1r13_kan25.rom", "r" );
	}
#endif
	if (fp)		// オープン成功時読込
	{
		dwReadSize = fread( mz1R13Kanji, 1, 131072, fp );
		fclose( fp );
	} else
		fprintf( stderr, "File open failed (fn: " FILENAME1 ")\n");
	
	//------*< 辞書ＲＯＭイメージファイル読み込み >*

	// ファイルが存在しない場合に備え全てFFで埋めとく 2003年03月01日
	memset(mz1R13Dic, 0xff, 16384);

	// ファイル「1R13DIC.ROM」を読込用でオープン
#define FILENAME2 "mz_1r13_dic.rom"
	fp = fopen( FILENAME2, "r" );
#ifdef EMZ2000_PLGDIR
	if (!fp) {
		char tpath[PATH_MAX];
		strncpy( tpath, EMZ2000_PLGDIR "/", PATH_MAX );
		tpath[PATH_MAX - 1] = '\0';
		strncat( tpath, FILENAME2, PATH_MAX - strlen(tpath) - 1 );
		fp = fopen( tpath, "r" );
	}
#endif
#ifdef EMZ2000_LIBDIR
	if (!fp) {
		char tpath[PATH_MAX];
		strncpy( tpath, EMZ2000_LIBDIR "/", PATH_MAX );
		tpath[PATH_MAX - 1] = '\0';
		strncat( tpath, FILENAME2, PATH_MAX - strlen(tpath) - 1 );
		fp = fopen( tpath, "r" );
	}
#endif

	if (fp)	// オープン成功時読込
	{
		dwReadSize = fread( mz1R13Dic, 1, 16384, fp );
		fclose( fp );
	} else
		fprintf( stderr, "File open failed (fn: " FILENAME2 ")\n");


	// I/O マッピングアドレス設定：B8H
	dwBase = 0xb8 ;
	mz1R13EBIH.ioMapper.ucStartPort = (UCHAR)dwBase ;

	address = 0 ;
	mz1R13Bank = 1;

	return	TRUE ;
}

BOOL
WINAPI
ReIntializeBoard( 
	VOID
	)
{
	address = 0 ;
	mz1R13Bank = 1;
	return	TRUE ;
}


VOID
WINAPI
FinalizeBoard( 
	VOID 
	)
{
	return ;
}

VOID
WINAPI
ReportCpuClock( 
	int clk
	)
{
	return ;
}

VOID
WINAPI
InfomationDialog( 
	HWND	hWnd 
	)
{
	return ;
}

BOOL
WINAPI
BoardInfomationService( 
	PEMUZ2000EBIH *info,
	DWORD id
	)
{

	*info = &mz1R13EBIH ;
	gId = id ;
	return	TRUE ;
}

#ifdef EMUZ2000EX
BOOL WINAPI StateSaveOutputDataEx( int *ver, DWORD *size, PUCHAR *buf )
{
	static unsigned int data[2];

	data[0] = address;
	data[1] = mz1R13Bank;

	*size = sizeof(data);
	*buf = (void *)data;
	*ver = 1;				/* Data Format Ver.1 */
	return TRUE;
}

BOOL WINAPI StateSaveInputDataEx( int ver, DWORD size, PUCHAR buf )
{
	unsigned int *data = (void *)buf;
	unsigned int tdata[2] = { 0, 0 };	/* for automatic size calculation */

	if (ver > 1)				/* Newer Data Format Ver. */
		return FALSE;
	if (size != sizeof(tdata))
		return FALSE;
	address    = (USHORT)data[0];
	mz1R13Bank = (UCHAR)data[1];
	return TRUE;
}
#else
BOOL
WINAPI
StateSaveOutputData( 
	DWORD *size,
	PUCHAR buf
	)
{
	*size = 0 ;
	buf = NULL ;
	return	TRUE ;
}

BOOL
WINAPI
StateSaveInputData( 
	DWORD size,
	PUCHAR buf
	)
{
	return	TRUE ;
}
#endif

/*
	Local Variables:
	mode:c++
	c-set-style:"k&r"
	c-basic-offset:4
	tab-width:4
	End:
*/
