/* File: uigtk_messagebox.c

	MZ-2000/2200/80B/B2 Emulator "emz2000" (gtk/messagebox)

	Copyright (C) 2003-2015 Toshio Fukui
	This file is part of the emz2000 software.
	See copyright notice in the COPYING file.
*/

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <gtk/gtk.h>

#include "common.h"
#include "mz2000.h"
#include "uigtk.h"

static GtkWidget *uigtk_messagebox_w;
static int uigtk_messagebox_id = UIGTK_IDCANCEL;

void uigtk_messagebox_ok( GtkWidget *w )
{
	gtk_widget_destroy( uigtk_messagebox_w );
	uigtk_messagebox_id = UIGTK_IDOK;
	return;
}

void uigtk_messagebox_cancel( GtkWidget *w )
{
	gtk_widget_destroy( uigtk_messagebox_w );
	uigtk_messagebox_id = UIGTK_IDCANCEL;
	return;
}

void uigtk_messagebox_yes( GtkWidget *w )
{
	gtk_widget_destroy( uigtk_messagebox_w );
	uigtk_messagebox_id = UIGTK_IDYES;
	return;
}

void uigtk_messagebox_no( GtkWidget *w )
{
	gtk_widget_destroy( uigtk_messagebox_w );
	uigtk_messagebox_id = UIGTK_IDNO;
	return;
}

int uigtk_messagebox_destroy( GtkWidget *widget, gpointer *data )
{
	gtk_main_quit();
	return FALSE;
}

int uigtk_messagebox( const char *message, const char *title, unsigned int attr )
{
	GtkMessageType icontype;
	GtkWidget *button_ok, *button_cancel;
	GtkWidget *button_yes, *button_no;

	icontype = GTK_MESSAGE_OTHER;
	if (attr & UIGTK_ICON_INFO)
		icontype = GTK_MESSAGE_INFO;
	if (attr & UIGTK_ICON_WARNING)
		icontype = GTK_MESSAGE_WARNING;
	if (attr & UIGTK_ICON_QUESTION)
		icontype = GTK_MESSAGE_QUESTION;
	if (attr & UIGTK_ICON_ERROR)
		icontype = GTK_MESSAGE_ERROR;
	uigtk_messagebox_id = UIGTK_IDCANCEL;
	uigtk_messagebox_w = gtk_message_dialog_new( GTK_WINDOW(uigtk_window),
		GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
		icontype,
		GTK_BUTTONS_NONE,
		(const gchar *)message, NULL );
	g_signal_connect( uigtk_messagebox_w,
		"destroy", G_CALLBACK(uigtk_messagebox_destroy), NULL );
	gtk_window_set_title( GTK_WINDOW(uigtk_messagebox_w), title );

	/* button YES */
	if (attr & UIGTK_BUTTON_YES) {
		button_yes = gtk_button_new_with_label("YES");
		g_signal_connect( button_yes,
			"clicked", G_CALLBACK(uigtk_messagebox_yes), uigtk_messagebox_w );
		gtk_box_pack_start( GTK_BOX(gtk_dialog_get_action_area(GTK_DIALOG(uigtk_messagebox_w))),
			button_yes, TRUE, TRUE, 0 );
		if (attr & UIGTK_BUTTON_DEFYES) {
			gtk_widget_set_can_default( button_yes, TRUE );
			gtk_widget_grab_default( button_yes );
		}
		gtk_widget_show( button_yes );
	} else
		button_yes = NULL;
	/* button NO */
	if (attr & UIGTK_BUTTON_NO) {
		button_no = gtk_button_new_with_label("NO");
		g_signal_connect( button_no,
			"clicked", G_CALLBACK(uigtk_messagebox_no), uigtk_messagebox_w );
		gtk_box_pack_start( GTK_BOX(gtk_dialog_get_action_area(GTK_DIALOG(uigtk_messagebox_w))),
			button_no, TRUE, TRUE, 0 );
		if (attr & UIGTK_BUTTON_DEFNO) {
			gtk_widget_set_can_default( button_no, TRUE );
			gtk_widget_grab_default( button_no );
		}
		gtk_widget_show( button_no );
	} else
		button_no = NULL;
	/* button OK */
	if (attr & UIGTK_BUTTON_OK) {
		button_ok = gtk_button_new_with_label("OK");
		g_signal_connect( button_ok,
			"clicked", G_CALLBACK(uigtk_messagebox_ok), uigtk_messagebox_w );
		gtk_box_pack_start( GTK_BOX(gtk_dialog_get_action_area(GTK_DIALOG(uigtk_messagebox_w))),
			button_ok, TRUE, TRUE, 0 );
		if (attr & UIGTK_BUTTON_DEFOK) {
			gtk_widget_set_can_default( button_ok, TRUE );
			gtk_widget_grab_default( button_ok );
		}
		gtk_widget_show( button_ok );
	} else
		button_ok = NULL;
	/* button Cancel */
	if (attr & UIGTK_BUTTON_CANCEL) {
		button_cancel = gtk_button_new_with_label("Cancel");
		g_signal_connect( button_cancel,
			"clicked", G_CALLBACK(uigtk_messagebox_cancel), uigtk_messagebox_w );
		gtk_box_pack_start( GTK_BOX(gtk_dialog_get_action_area(GTK_DIALOG(uigtk_messagebox_w))),
			button_cancel, TRUE, TRUE, 0 );
		if (attr & UIGTK_BUTTON_DEFCANCEL) {
			gtk_widget_set_can_default( button_cancel, TRUE );
			gtk_widget_grab_default( button_cancel );
		}
		gtk_widget_show( button_cancel );
	} else
		button_cancel = NULL;

	gtk_widget_show( uigtk_messagebox_w );
	gtk_main();
	return uigtk_messagebox_id;
}

/*
	Local Variables:
	mode:c++
	c-set-style:"k&r"
	c-basic-offset:8
	tab-width:8
	End:
*/
